package org.book.client;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.helpers.Loader;
import org.books.data.Book;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/>
 * Modul: Business-Tier (EJB 3)<br/>
 * Beschreibung:<br/>
 * Diese Klasse ist eine Utility-Klasse und fuellt die Datenbank mit Werten.
 * </p>
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author   Marc Bouquet ( bouquet dot ch, Switzerland )
 * @version  1.0 - 19.06.2007
 */
public class DataLoaderBean implements Serializable
{
  /**
   * Default serial version uid
   */
  private static final long serialVersionUID = 1L;

  /**
   * constant for the logging
   */
  private transient Logger LOG = Logger.getLogger( DataLoaderBean.class );

  /** 
   * The name of the book data file. 
   */
  private static final String BOOK_DATA_FILE = "/books.csv";

  //-----------------------------------------------------------------------------
  // C O N S T R U C T O R S
  //-----------------------------------------------------------------------------

  /**
   * Standard Konstruktor
   */
  public DataLoaderBean()
  {
    PropertyConfigurator.configure( Loader.getResource( "log4j.properties" ) );
    PropertyConfigurator.configureAndWatch( "log4j.properties" );
    if ( LOG.isInfoEnabled() )
    {
      LOG.info( "Startet das laden der Datenbank" );
    }
    List<Book> list = readData();
    if ( list != null && list.size() > 0 )
    {
      load( "bookload", list );
    }
    if ( LOG.isInfoEnabled() )
    {
      LOG.info( "Laden der Datenbank wurde beendet..." );
    }
  }

  /**
   * Load testdata into the eadj database
   * @param ctx String context
   * @param list List<Book>
   */
  public void load( String ctx, List<Book> list )
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( ctx );
    EntityManager em = emf.createEntityManager();

    try
    {
      // load books
      for ( Iterator iter = list.iterator(); iter.hasNext(); )
      {
        em.getTransaction().begin();
        Book book = (Book) iter.next();
        em.persist( book );
        em.getTransaction().commit();
        System.out.println( book.toString() );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /** 
   * Reads the book data. 
   */
  private List<Book> readData()
  {
    System.out.println( "call readData ..." );
    try
    {
      List<Book> list = new ArrayList<Book>();
      BufferedReader in = new BufferedReader( new InputStreamReader( getClass().getResourceAsStream( BOOK_DATA_FILE ) ) );
      DateFormat df = new SimpleDateFormat( "yyyy-MM-dd" );
      while ( true )
      {
        String line = in.readLine();
        if ( line == null )
        {
          break;
        }
        String[] tokens = line.split( ";" );
        Book book = new Book();
        book.setIsbn( tokens[0] );
        book.setTitle( tokens[1] );
        book.setAuthors( tokens[2] );
        book.setPublisher( tokens[3] );
        book.setDate( df.parse( tokens[4] ) );
        book.setBinding( tokens[5] );
        book.setPages( new Integer( tokens[6] ) );
        book.setPrice( new Double( tokens[7] ) );
        list.add( book );
      }
      System.out.println( list.size() );
      return list;
    }
    catch ( Exception ex )
    {
      System.err.println( ex.toString() );
      return null;
    }
  }

  //-----------------------------------------------------------------------------
  // M A I N - M E T H O D
  //-----------------------------------------------------------------------------

  /**
   * Laden der Werte in die Datenbank
   * @param args no arguments required
   */
  public static void main( String[] args )
  {
    new DataLoaderBean();
  }
}
